from netCDF4 import Dataset 
import numpy as np
import xarray as xr
import pandas as pd
import math
import glob
import netCDF4 as nc
import matplotlib.pyplot as plt
import cartopy.crs as ccrs
import cartopy.feature as cfeature
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
import cmaps
from matplotlib import rcParams
import cartopy.feature as cf
import matplotlib.patches as mpatchesv
import heapq
from scipy import interpolate 
import scipy.stats as stats
from scipy.stats import pearsonr
from scipy.interpolate import griddata
from scipy.stats import ttest_ind
import matplotlib.gridspec as gridspec
from matplotlib.patches import Rectangle
import matplotlib.image as im

def snapcmap(imagename):
    image=im.imread(imagename)
    color=image[round(np.shape(image)[0]/2),:,:]
    icamp=ListedColormap(color)
    return icamp

def griddata_myself(lat, lon , data, lat_new, lon_new):
    xy = np.array([[y, x] for x in lat for y in lon])
    data_points = np.reshape(data, (len(lat)*len(lon), 1), order='C')
    grid_x, grid_y = np.meshgrid(lon_new, lat_new)
    data_grid = griddata(xy, data_points, (grid_x, grid_y), method='linear').squeeze()
    return data_grid

def T_test(data_fc,data_md,data3,data4):
    std1=np.zeros((data3.shape[1],data3.shape[2]))
    std2=np.zeros((data4.shape[1],data4.shape[2]))    
    for i in range(data3.shape[1]):
        for j in range(data3.shape[2]):
            std1[i,j]=np.std(data3[:,i,j])
            std2[i,j]=np.std(data4[:,i,j])
    z=np.zeros((data_fc.shape[0],data_fc.shape[1]))
    for i in range(data3.shape[1]):
        for j in range(data3.shape[2]):
            data_math1=std1[i,j]**2/(50)
            data_math2=std2[i,j]**2/(20)

            z[i,j]=(data_fc[i,j]-data_md[i,j])/np.sqrt(data_math1+data_math2)
            z1=abs(z)
    return z1 

f1=nc.Dataset('Obsveredmean.trend.SST.nc')
lat_obs=f1.variables['lat'][:].squeeze()
lon_obs=f1.variables['lon'][:].squeeze()

obs_sst=f1.variables['sst'][:,:].squeeze()
obs_area=f1.variables['area'][:,:].squeeze()
##########################################################################################CMIP6

f1=nc.Dataset('CMIP6mean.trend.SST.nc')
lon_cmip6=f1.variables['lon'][:].squeeze()
lat_cmip6=f1.variables['lat'][:].squeeze()

cmip6_sst=f1.variables['sst'][:,:].squeeze()
cmip6_area=f1.variables['area'][:,:].squeeze()

##########################################################################################FC_MD
f1=nc.Dataset('cesm2.FC.ensmean.trend.SST.nc')
lat_cesm=f1.variables['lat'][:].squeeze()
lon_cesm=f1.variables['lon'][:].squeeze()

fc_sst=f1.variables['sst'][:,:].squeeze()
fc_area=f1.variables['area'][:,:].squeeze()

f1=nc.Dataset('cesm2.MD.ensmean.trend.SST.nc')
md_sst=f1.variables['sst'][:,:].squeeze()
md_area=f1.variables['area'][:,:].squeeze()

f1=nc.Dataset('cesm2.FC-MD.ensmean.trend.SST.nc')
fc_md_sst=f1.variables['sst'][:,:].squeeze()
fc_md_area=f1.variables['area'][:,:].squeeze()


f1=nc.Dataset('cesm2.FC-MD.ensmean.trend.Taux_Tauy.nc')
du_slope=f1.variables['Taux'][:,:].squeeze()
dv_slope=f1.variables['Tauy'][:,:].squeeze()

data=[obs_sst,cmip6_sst,fc_sst,md_sst,fc_md_sst]
area=[obs_area,cmip6_area,fc_area,md_area,fc_md_area]

u_slope=du_slope
v_slope=dv_slope

lon=[lon_obs,lon_cmip6,lon_cesm,lon_cesm,lon_cesm]
lat=[lat_obs,lat_cmip6,lat_cesm,lat_cesm,lat_cesm]

range_number1=[-0.2,-0.1,0.0,0.1,0.2]
range_number4=[-0.1,-0.05,0.0,0.05,0.1]
levels1=np.arange(-0.2,0.21,0.02)
levels4 =np.arange(-0.1,0.11,0.01)

#######################################################################################
label11=['$^\circ$C/decade','$^\circ$C/decade','$^\circ$C/decade','$^\circ$C/decade','$^\circ$C/decade']

title1=['(a) Mean Observed ','(b) CMIP6-MMM' ,'(c) FC'  ,'(d) MD' ,'(e) FC-MD']
cmap_color = cmaps.BlueDarkRed18
range_number=[range_number1,range_number1,range_number1,range_number1,range_number4]
levels_number=[levels1,levels1,levels1,levels1,levels4]

cax1=[[0.441, 0.736, 0.010, 0.2],[0.874, 0.736, 0.010, 0.2],
      [0.441, 0.46, 0.010, 0.2],[0.874, 0.46, 0.010, 0.2],
      [0.658, 0.182, 0.010, 0.2]]

uv_speed=[1]
label_title=[f'{1} '+'x10$^{-3}$ N'+"\u00B7"+'m$^{-2}$/decade']
color_speed=['k','k','k','k']
alpha11=[1,1,1,1]
num=[3.5]
number=[321,322,323,324]
region = [120,290,-30,30]  

linewidth_num=[3.0,0,0,0,0]
font_title={'family':'Arial',
            'weight':'normal',
            'size':16}
config = {"font.family":'Arial', "font.size": 16, "mathtext.fontset":'stix'}

fig = plt.figure(figsize=(20,12),dpi=500)
gs = gridspec.GridSpec(3, 4) # 创立2 * 6 网格
gs.update(wspace=0.8)
for i in range(2):
    for j in range(2):
        proj=ccrs.PlateCarree(central_longitude=180)  
        ax = fig.add_subplot(gs[i, 2*j:2+2*j], projection=proj)

        ax.spines ['top'] .set_linewidth(1.3)
        ax.spines ['right'] .set_linewidth(1.3)
        ax.spines ['bottom'] .set_linewidth(1.3)
        ax.spines ['left'] .set_linewidth(1.3)
        ax.spines['geo'].set_linewidth(1.3)
        land = cfeature.NaturalEarthFeature('physical', 'land', scale='50m', facecolor = cfeature.COLORS['land'])
        ax.add_feature(land, facecolor = 'lightgray',zorder=1) ##这两步是把海岸线中的陆地填上颜色
        ax.coastlines(linewidth = 1.2) 
        ax.set_extent(region, crs = ccrs.PlateCarree())
        ax.set_xticks([120,150, 180, 210, 240,270],crs = ccrs.PlateCarree(),)
        ax.set_yticks([-30,-15,0, 15,30],crs = ccrs.PlateCarree(),)

        lon_formatter = LongitudeFormatter(zero_direction_label=True)
        lat_formatter = LatitudeFormatter()
        ax.xaxis.set_major_formatter(lon_formatter)
        ax.yaxis.set_major_formatter(lat_formatter)
        ax.xlabel_style = {'font.family':'Arial','size': 28, 'color': 'k'}  #####label字体
        ax.ylabel_style = {'font.family':'Arial','size': 28, 'color': 'k'}
        ax.tick_params(direction = 'out',size=8,width=1.2)##刻度长短

        title11 =title1[2*i+j]
        ax.set_title(title11,loc='left',fontsize=21,fontdict={'fontname': 'Arial'})
        levels_s =levels_number[0]
        cmap_color=cmap_color#color1[0]
        lon11=lon[2*i+j]
        lat11=lat[2*i+j]
        data_data=data[2*i+j]
        cs = ax.contourf(lon11, lat11, data_data, levels=levels_s,zorder=0,
                         transform=ccrs.PlateCarree(),cmap = cmap_color,extend = 'both') 
        sig2 = ax.contourf(lon11, lat11,area[2*i+j],colors='none',hatches=['..',None],transform=ccrs.PlateCarree(),zorder=0)

        start_point = (lon_obs[120], lat_obs[85])
        RE=Rectangle(start_point,lon_obs[179]-lon_obs[120],lat_obs[94]-lat_obs[85],linewidth=linewidth_num[2*i+j],linestyle='-' ,zorder=2,\
                  edgecolor='blue',facecolor='none', transform=ccrs.PlateCarree()) #25表示纬度间隔25
        ax.add_patch(RE)  
        start_point = (lon_obs[180], lat_obs[85])
        RE1=Rectangle(start_point,lon_obs[229]-lon_obs[180],lat_obs[94]-lat_obs[85],linewidth=linewidth_num[2*i+j],linestyle='-' ,zorder=2,\
                  edgecolor='orange',facecolor='none', transform=ccrs.PlateCarree()) #25表示纬度间隔25
        ax.add_patch(RE1)  

        start_point = (lon_obs[230], lat_obs[85])
        RE2=Rectangle(start_point,lon_obs[280]-lon_obs[230],lat_obs[94]-lat_obs[85],linewidth=linewidth_num[2*i+j],linestyle='-' ,zorder=2,\
                  edgecolor='red',facecolor='none', transform=ccrs.PlateCarree()) #25表示纬度间隔25            
        ax.add_patch(RE2)  
        rcParams.update(config)

        cax11=cax1[2*i+j]
        cax = plt.axes(cax11)  #右 下  长 宽
        
        font_title={'family':'Arial',
            'weight':'normal',
            'size':16}
            
        cbar = fig.colorbar(cs,cax=cax,cmap = cmap_color, extend='both', orientation='vertical')#
        label1=label11[2*i+j]
        cbar.set_label(label1,font=font_title,labelpad=1)
        cbar.ax.tick_params(direction = 'out',size=1.4)##刻度长短
        range_numbers=range_number[0]
        cbar.set_ticks(range_numbers,font=font_title)

i=4
proj=ccrs.PlateCarree(central_longitude=180)  
#ax = fig.add_subplot(number[i], projection=proj)
ax = fig.add_subplot(gs[2, 1:3], projection=proj)

ax.spines ['top'] .set_linewidth(1.3)
ax.spines ['right'] .set_linewidth(1.3)
ax.spines ['bottom'] .set_linewidth(1.3)
ax.spines ['left'] .set_linewidth(1.3)
ax.spines['geo'].set_linewidth(1.3)
land = cfeature.NaturalEarthFeature('physical', 'land', scale='50m', facecolor = cfeature.COLORS['land'])
ax.add_feature(land, facecolor = 'lightgray',zorder=1) 
ax.coastlines(linewidth = 1.2)  
ax.set_extent(region, crs = ccrs.PlateCarree())  
ax.set_xticks([120,150, 180, 210, 240,270],crs = ccrs.PlateCarree(),)
ax.set_yticks([-30,-15,0, 15,30],crs = ccrs.PlateCarree(),)

lon_formatter = LongitudeFormatter(zero_direction_label=True)
lat_formatter = LatitudeFormatter()
ax.xaxis.set_major_formatter(lon_formatter)
ax.yaxis.set_major_formatter(lat_formatter)
ax.xlabel_style = {'font.family':'Arial','size': 28, 'color': 'k'} 
ax.ylabel_style = {'font.family':'Arial','size': 28, 'color': 'k'}

ax.tick_params(direction = 'out',size=8,width=1.2)##刻度长短
title11 =title1[i]
ax.set_title(title11,loc='left',fontsize=21,fontdict={'fontname': 'Arial'})

levels_s =levels_number[i]
cmap_color=cmap_color
lon11=lon[i]
lat11=lat[i]
data_data=data[i]
cs = ax.contourf(lon11, lat11, data_data, levels=levels_s,zorder=0,transform=ccrs.PlateCarree(),
                 cmap = cmap_color,extend = 'both')
sig2 = ax.contourf(lon11, lat11,area[i],colors='none',hatches=['..',None],transform=ccrs.PlateCarree(),zorder=0)

step1=1
step2=1
u_slope_1=u_slope
v_slope_1=v_slope
Q = ax.quiver(lon_cesm[::step2], lat_cesm[::step1],u_slope_1[::step1,::step2], v_slope_1[::step1,::step2],transform=ccrs.PlateCarree(),zorder=0,
 scale=num[0],units='inches', width=0.02, headwidth=2.4,color='k',alpha=1.0,regrid_shape=14)
qk= ax.quiverkey(Q,  0.63, 1.06, U=uv_speed[0], label=label_title[0],
                 angle=0,color=color_speed[0], coordinates='axes',
             labelpos='E',labelcolor=color_speed[0],fc=color_speed[0],labelsep=0.05,alpha=alpha11[0],
              fontproperties={'size': 19,'family':'Arial'})

rcParams.update(config)
cax11=cax1[4]
cax = plt.axes(cax11)
cbar = fig.colorbar(cs,cax=cax,cmap = cmap_color, extend='both', orientation='vertical')#
label1=label11[4]

font_title={'family':'Arial',
            'weight':'normal',
            'size':16}
    
cbar.set_label(label1,font=font_title,labelpad=1)
cbar.ax.tick_params(direction = 'out',size=1.6)##刻度长短
range_numbers=range_number[4]
cbar.set_ticks(range_numbers,font=font_title)

plt.tight_layout()
fig.subplots_adjust(left=0.099, bottom=0.169, right=0.869, top=0.948, wspace=0.315, hspace=0.247)
print('end')
